import os
from requests import get
from jovian.utils.constants import API_URL, ISSUES_MSG
from jovian._version import __version__
from jovian.utils.credentials import get_guest_key, read_api_key_opt
from jovian.utils.logger import log
from jovian.utils.rcfile import set_notebook_slug, get_rcdata, rcfile_exists


def _u(path):
    """Make a URL from the path"""
    return API_URL + path


def _msg(res):
    try:
        data = res.json()
        if 'errors' in data and len(data['errors'] > 0):
            return data['errors'][0]['message']
        if 'message' in data:
            return data['message']
        if 'msg' in data:
            return data['msg']
    except:
        if res.text:
            return res.text
        return 'Something went wrong'


def _pretty(res):
    """Make a human readable output from an HTML response"""
    return '(HTTP ' + str(res.status_code) + ') ' + _msg(res)


def _h(fresh):
    """Create a header to provide library metadata"""
    api_key, _ = read_api_key_opt()

    headers = {"x-jovian-source": "library",
               "x-jovian-library-version": __version__,
               "x-jovian-command": "clone" if fresh else "pull",
               "x-jovian-guest": get_guest_key()}

    if api_key is not None:
        headers["Authorization"] = "Bearer " + api_key

    return headers


def _v(version):
    """Create version query parameter string"""
    if version is not None:
        return "?gist_version=" + str(version)
    return ""


def get_gist(slug, version, fresh):
    """Download a gist"""
    url = _u('/gist/' + slug + _v(version))
    res = get(url, headers=_h(fresh))
    if res.status_code == 200:
        return res.json()['data']
    raise Exception('Failed to retrieve Gist: ' + _pretty(res))


def post_clone_msg(title):
    return """Cloned successfully to '{}'. 

Next steps:
$ cd {}   # Enter the directory
$ jovian install     # Install dependencies
$ conda activate <env_name> # Activate environment
$ jupyter notebook   # Start Jupyter

Replace <env_name> with the name of your environment (without the '<' & '>')
Jovian uses Anaconda ( https://conda.io/ ) under the hood, 
so please make sure you have it installed and added to path. 
* If you face issues with `jovian install`, try `conda env update`.
* If you face issues with `conda activate`, try `source activate <env_name>` 
  or `activate <env_name>` to activate the virtual environment.

{}
""".format(title, title, ISSUES_MSG)


def clone(slug, version=None, fresh=True):
    """Download the files for a gist"""
    # Print issues link
    log(ISSUES_MSG)

    # Download gist metadata
    ver_str = '(version ' + version + ')' if version else ''
    log('Fetching ' + slug + " " + ver_str + "..")
    gist = get_gist(slug, version, fresh)
    title = gist['title']

    # If fresh clone, create directory
    if fresh:
        if os.path.exists(title):
            i = 1
            while os.path.exists(title + '-' + str(i)):
                i += 1
            title = title + '-' + str(i)
        if not os.path.exists(title):
            os.makedirs(title)
        os.chdir(title)

    # Download the files
    log('Downloading files..')
    for f in gist['files']:
        with open(f['filename'], 'wb') as fp:
            fp.write(get(f['rawUrl']).content)

        # Create .jovianrc for a fresh clone
        if fresh and f['filename'].endswith('.ipynb'):
            set_notebook_slug(f['filename'], slug)

    # Print success message and instructions
    if fresh:
        log(post_clone_msg(title))
    else:
        log('Files dowloaded successfully in current directory')


RCFILE_NOTFOUND = "Could not detect '.jovianrc' file. Make sure you are running 'jovian pull' inside a directory cloned with 'jovian clone'."


def pull(slug=None, version=None):
    """Get the latest files associated with the current gist"""
    # If a slug is provided, just use that
    if slug:
        clone(slug, version, fresh=False)
        return

    # Check if .jovianrc exists
    if not rcfile_exists():
        log(RCFILE_NOTFOUND, error=True)
        return

    # Get list of notebooks
    nbs = get_rcdata()['notebooks']
    for fname in nbs:
        # Get the latest files for each notebook
        clone(nbs[fname]['slug'], version, fresh=False)



import os
from os.path import basename
from jovian._version import __version__
from time import sleep
from jovian.utils.anaconda import upload_conda_env, CondaError
from jovian.utils.pip import upload_pip_env
from jovian.utils.api import (create_gist_simple, upload_file, get_gist_access,
                              post_block, commit_records, post_slack_message)
from jovian.utils.logger import log
from jovian.utils.constants import WEBAPP_URL, FILENAME_MSG, RC_FILENAME
from jovian.utils.jupyter import set_notebook_name, in_notebook, save_notebook, get_notebook_name
from jovian.utils.rcfile import get_notebook_slug, set_notebook_slug, make_rcdata

set_notebook_name()

_current_slug = None
_data_blocks = []


def reset():
    """Reset the tracked hyperparameters & metrics (for a fresh experiment)"""
    global _current_slug
    global _data_blocks
    _current_slug = None
    _data_blocks = []


def commit(secret=False, nb_filename=None, files=[], capture_env=True,
           env_type='conda', notebook_id=None, create_new=None, artifacts=[]):
    """Commits a Jupyter Notebook with its environment to Jovian.

    Saves the checkpoint of the notebook, capture the required dependencies from the python environment and uploads the notebook, env file, additional files like scripts, csv etc. to https: // www.jvn.io . Capturing the python environment ensures that the notebook can be reproduced and
    executed easily using the ** *{links to reprodue notebooks}.***

    Args:
        secret(bool, optional): Create a secret notebook on Jovian, which is only
            accessible via the link, and is not visible on the owner's public profile. By default,
            committed notebooks are public and visible on the owner's profile.

        nb_filename(string, optional): The filename of the jupyter notebook(including
            the .ipynb extension). This is detected automatically in most cases, but in
            certain environments like Jupyter Lab, the detection may fail and the filename
            needs to be provided using this argument.

        files(array, optional): Any additional scripts(.py files), CSVs that are required to
            run the notebook. These will be available in the files tab on Jovian .

        capture_env(bool, optional): If `True`, the Python environment(python version,
            libraries etc.) are captured and uploaded along with the notebook.

        env_type(string, optional): The type of environment to be captured. Allowed options are
            'conda' and 'pip'.

        notebook_id(string, optional): If you wish to update an existing notebook owned by you,
            you can use this argument to provide the base64 ID(present in the URL) of an notebook
            hosted on Jovian . In most cases, this argument is not required, and the library
            can automatically infer whether you are looking to update an existing notebook or create
            a new one.

        create_new(bool, optional): If set to True, doesn't update the existing notebook on
            Jovian(if one is detected). Instead, it creates a new notebook when commit is called.

        artifacts(array, optional): Any outputs files or artifacts generated from the modeling processing.
            This can include model weights/checkpoints, generated CSVs, images etc.

    .. attention: :
        Pass notebook's name to nb_filename in ceratin environments like Jupyter Lab, password protected notebooks as sometimes it may fail to detect automatically in these environments.

    """
    global _current_slug
    global _data_blocks

    # Check if we're in a Jupyter environment
    if not in_notebook():
        log('Failed to detect Juptyer notebook. Skipping..', error=True)
        return

    # Save the notebook (uses Javascript, doesn't work everywhere)
    log('Saving notebook..')
    save_notebook()
    sleep(1)

    # Get the filename of the notebook (if not provided)
    if nb_filename is None:
        nb_filename = get_notebook_name()

    # Exit with help message if filename wasn't detected (or provided)
    if nb_filename is None:
        log(FILENAME_MSG)
        return

    # Check whether to create a new gist, or update an old one
    if not create_new and notebook_id is None:
        # First preference to the in-memory slug variable
        if _current_slug is not None:
            notebook_id = _current_slug
        else:
            notebook_id = get_notebook_slug(nb_filename)

    # Check if the current user can push to this slug
    if notebook_id is not None:
        gist_access = get_gist_access(notebook_id)
        if not gist_access['write']:
            notebook_id = None

    # Log whether this is an update or creation
    if notebook_id is None:
        log('Creating a new notebook on ' + WEBAPP_URL)
    else:
        log('Updating notebook "' + notebook_id + '" on ' + WEBAPP_URL)

    # Upload the notebook & create/update the gist
    res = create_gist_simple(nb_filename, notebook_id, secret)
    if res is None:
        return

    # Extract slug and owner from created gist
    slug, owner, version = res['slug'], res['owner'], res['version']

    # Set/update the slug information
    _current_slug = slug
    set_notebook_slug(nb_filename, slug)

    # Save & upload environment
    if capture_env:
        log('Capturing environment..')

        if env_type == 'conda':
            # Capture conda environment
            try:
                upload_conda_env(gist_slug=slug, version=version)
            except CondaError as e:
                log(str(e), error=True)

        elif env_type == 'pip':
            # Capture pip environment
            try:
                upload_pip_env(gist_slug=slug, version=version)
            except Exception as e:
                log(str(e), error=True)

    # Upload additional files
    if files and len(files) > 0:
        log('Uploading additional files..')

        # Upload each file
        for fname in files:
            if os.path.exists(fname) and not os.path.isdir(fname):
                try:
                    with open(fname, 'rb') as f:
                        file = (basename(fname), f)
                        upload_file(gist_slug=slug, file=file, version=version)
                except Exception as e:
                    log(str(e), error=True)
            elif os.path.isdir(fname):
                log('Ignoring directory "' + fname + '"', error=True)
            else:
                log('Ignoring "' + fname + '" (not found)', error=True)

    # Upload artifacts
    if artifacts and len(artifacts) > 0:
        log('Uploading artifacts..')

        # Upload each artifact
        for fname in artifacts:
            if os.path.exists(fname) and not os.path.isdir(fname):
                try:
                    with open(fname, 'rb') as f:
                        file = (basename(fname), f)
                        upload_file(gist_slug=slug, file=file,
                                    version=version, artifact=True)
                except Exception as e:
                    log(str(e), error=True)
            elif os.path.isdir(fname):
                log('Ignoring directory "' + fname +
                    '". Please include files directly', error=True)
            else:
                log('Ignoring "' + fname + '" (not found)', error=True)

    # Record metrics & hyperparameters
    if len(_data_blocks) > 0:
        log('Recording metrics & hyperparameters..')
        commit_records(slug, _data_blocks, version)

    # Print commit URL
    log('Committed successfully! ' + WEBAPP_URL +
        "/" + owner['username'] + "/" + slug)


def log_hyperparams(data, verbose=True):
    """Record hyperparameters for the current experiment

    Args:
        data(dict): A python dict or a array of dicts to be recorded as hyperparmeters.

        verbose(bool, optional): By default it prints the acknowledgement, you can remove this by setting the argument to False.

    Example: :

        hyperparmas = {
            arch_name: "cnn_1"
            lr: .001
        }
        jovian.log_hyperparams(hyperparams)
    """
    global _data_blocks
    res = post_block(data, 'hyperparams')
    _data_blocks.append(res['tracking']['trackingSlug'])
    if verbose:
        log('Hyperparameters logged.')


def log_metrics(data, verbose=True):
    """Record metrics for the current experiment

    Args:
        data(dict): A python dict or a array of dicts to be recorded as metrics.

        verbose(bool, optional): By default it prints the acknowledgement, you can remove this by setting the argument to False.

    Example: :

        metrics = {
            epoch: 1
            train_loss: .5
            val_loss: .3
            acc: .94
        }
        jovian.log_metrics(metrics)
    """
    global _data_blocks
    res = post_block(data, 'metrics')
    _data_blocks.append(res['tracking']['trackingSlug'])
    if verbose:
        log('Metrics logged.')


def notify(data, verbose=False, safe=False):
    """Sends the data to Slack connected to Jovian account

    Arguments:
        data: A dict or string to be pushed to Slack
    """
    res = post_slack_message(data=data, safe=safe)
    if verbose:
        if not res.get('errors'):
            log('message_sent:' + str(res.get('data').get('messageSent')))
        else:
            log(str(res.get('errors')[0].get('message')), error=True)



import os
from requests import get
from jovian.utils.constants import API_URL
from jovian.utils.credentials import get_guest_key, read_api_key_opt
from jovian.utils.logger import log
from jovian._version import __version__


class ApiError(Exception):
    """Error class for web API related Exceptions"""
    pass


def _u(path):
    """Make a URL from the path"""
    return API_URL + path


def _msg(res):
    try:
        data = res.json()
        if 'errors' in data and len(data['errors'] > 0):
            return data['errors'][0]['message']
        if 'message' in data:
            return data['message']
        if 'msg' in data:
            return data['msg']
    except:
        if res.text:
            return res.text
        return 'Something went wrong'


def _pretty(res):
    """Make a human readable output from an HTML response"""
    return '(HTTP ' + str(res.status_code) + ') ' + _msg(res)


def _h():
    """Create a header to provide library metadata"""
    api_key, _ = read_api_key_opt()

    headers = {"x-jovian-source": "library",
               "x-jovian-library-version": __version__,
               "x-jovian-command": "add-slack",
               "x-jovian-guest": get_guest_key()}

    if api_key is not None:
        headers["Authorization"] = "Bearer " + api_key

    return headers


def _v(version):
    """Create version query parameter string"""
    if version is not None:
        return "?gist_version=" + str(version)
    return ""


def add_slack():
    """prints instructions for connecting Slack, if Slack connection is not already present.
    if Slack is already connected, prints details about the workspace and the channel"""
    url = _u('/slack/integration_details')
    res = get(url, headers=_h())
    if res.status_code == 200:
        res = res.json()
        if not res.get('errors'):
            slack_account = res.get('data').get('slackAccount')
            log('Slack already connected. \nWorkspace: {}\nConnected Channel: {}'
                .format(slack_account.get('workspace'), slack_account.get('channel')))
        else:
            log(str(res.get('errors')[0].get('message')))
    else:
        raise ApiError('Slack trigger failed: ' + _pretty(res))



"""Utilities to read, write and manage the credentials file"""
import os
from getpass import getpass
import json
import stat
import shutil
import uuid
from uuid import UUID
from jovian.utils.logger import log
from jovian.utils.constants import WEBAPP_URL, API_KEY, GUEST_KEY


CREDS = {}

HOME = os.path.expanduser('~')
CONFIG_DIR = HOME + '/.jovian'
CREDS_FNAME = 'credentials.json'
CREDS_PATH = CONFIG_DIR + '/' + CREDS_FNAME


def config_exists():
    """Check if config directory exists"""
    return os.path.exists(CONFIG_DIR)


def init_config():
    """Create the config directory"""
    if not os.path.exists(CONFIG_DIR):
        os.makedirs(CONFIG_DIR)


def purge_config():
    """Remove the config directory"""
    return shutil.rmtree(CONFIG_DIR, ignore_errors=True)


def purge_creds():
    """Remove the config directory"""
    if os.path.isfile(CREDS_PATH):
        os.remove(CREDS_PATH)


def creds_exist():
    """Check if credentials file exits"""
    return os.path.exists(CREDS_PATH)


def read_creds():
    """Read the credentials file"""
    with open(CREDS_PATH, 'r') as f:
        try:
            return json.load(f)
        except ValueError:
            purge_creds()
            return {}


def write_creds(creds, update_cache=True):
    """Write the given credentials to file"""
    init_config()
    with open(CREDS_PATH, 'w') as f:
        json.dump(creds, f)
    os.chmod(CREDS_PATH, stat.S_IREAD | stat.S_IWRITE)

    if update_cache:
        _get_or_init_creds()


def _get_or_init_creds():
    global CREDS
    CREDS = read_creds() if creds_exist() else {}
    return CREDS


def _write_key(key, key_name):
    creds = _get_or_init_creds()
    if key_name in creds and creds[key_name] == key:
        return
    else:
        creds[key_name] = key
        write_creds(creds)


def write_guest_key(key):
    """Write the GUEST key to memory, and the credentials file"""
    _write_key(key, GUEST_KEY)


def write_api_key(key):
    """Write the API key to memory, and the credentials file"""
    _write_key(key, API_KEY)


def request_api_key():
    """Ask the user to provide the API key"""
    log("Please enter your API key (from " + WEBAPP_URL + " ):")
    api_key = getpass(prompt="API Key:")
    return api_key


def read_api_key_opt():
    """Read credentials file, or return None"""
    creds = _get_or_init_creds()
    api_key = creds[API_KEY] if API_KEY in creds else None

    return api_key, 'read'


def read_or_request_api_key():
    """Read credentials file, or ask the user for API Key, if required"""
    api_key, source = read_api_key_opt()

    if api_key is not None:
        return api_key, source
    else:
        return request_api_key(), 'request'


def _generate_guest_key():
    """Generate GUEST key"""
    return uuid.uuid4().hex


def _read_or_generate_guest_key():
    """Read credentials file, or generate GUEST key, if required"""
    creds = _get_or_init_creds()
    return creds[GUEST_KEY] if GUEST_KEY in creds else _generate_guest_key()


def _validate_guest_key(key):
    """Validate GUEST key"""
    try:
        val = UUID(key, version=4)
    except ValueError:
        return False
    return val.hex == key


def get_guest_key():
    if GUEST_KEY not in CREDS:
        key = _read_or_generate_guest_key()
        if not _validate_guest_key(key):
            key = _generate_guest_key()
        write_guest_key(key)
        return key

    return CREDS[GUEST_KEY]



import unittest
# import yaml
from jovian.utils.envfile import (check_error, extract_env_name, extract_env_packages,
                                  extract_package_from_line, extract_pip_packages,
                                  get_environment_dict, identify_env_file)

FILES_PREFIX = 'jovian/tests/resources/'     # change based on which dir you're running the tests in
# eg: for running only this file, change FILES_PREFIX = 'resources/'


class InstallUtilsTest(unittest.TestCase):

    def test_identify_env_file(self):
        env_fname = identify_env_file(env_fname=None)
        self.assertEqual(env_fname, 'environment.yml')

    def test_get_environment_dict(self):
        env_filename = FILES_PREFIX + 'environment.yml'
        expected = {'prefix': '/home/admin/anaconda3/envs/test-env',
                    'dependencies': ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                     {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
                    'channels': ['defaults'], 'name': 'test-env'}
        environment = get_environment_dict(env_fname=env_filename)

        self.assertDictEqual(environment, expected)

        with self.assertRaises(FileNotFoundError):
            get_environment_dict(env_fname='non-existent-file.yml')

        # with self.assertRaises(yaml.YAMLError):    # we're printing the exception, not raising it.
        #     get_environment_dict(env_fname=FILES_PREFIX + 'invalid-yaml-file.yml')

    def test_extract_env_name(self):
        env_filename = FILES_PREFIX + 'environment.yml'
        name = extract_env_name(env_fname=env_filename)
        name2 = extract_env_name(env_fname=FILES_PREFIX+'empty-yaml-file.yml')

        self.assertEqual(name, 'test-env')
        self.assertIsNone(name2)
        with self.assertRaises(FileNotFoundError):
            extract_env_name(env_fname='non-existent-file.yml')

    def test_extract_env_packages(self):
        env_filename = FILES_PREFIX + 'environment.yml'
        dep = extract_env_packages(env_fname=env_filename)
        dep2 = extract_env_packages(env_fname=FILES_PREFIX + 'empty-yaml-file.yml')

        self.assertListEqual(dep, ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                   'six==1.11.0', 'sqlite==2.0.0'])
        self.assertListEqual(dep2, [])
        with self.assertRaises(FileNotFoundError):
            extract_env_packages(env_fname='non-existent-file.yml')

    def test_extract_pip_packages(self):
        env_filename = FILES_PREFIX + 'environment.yml'
        env_filename2 = FILES_PREFIX + 'empty-yaml-file.yml'
        dep = extract_pip_packages(env_fname=env_filename)
        dep2 = extract_pip_packages(env_fname=env_filename2)

        self.assertListEqual(dep, ['six==1.11.0', 'sqlite==2.0.0'])
        self.assertListEqual(dep2, [])
        with self.assertRaises(FileNotFoundError):
            extract_env_packages(env_fname='non-existent-file.yml')

    def test_check_error(self):
        env_filename = FILES_PREFIX + 'environment.yml'
        packages = extract_env_packages(env_fname=env_filename)

        error_str = '''ResolvePackageNotFound: 
                        - mixpanel=1.11.0'''
        error_str2 = '''UnsatisfiableError: 
                                - sigmasix=1.91.0'''
        error_str3 = '''AnyOtherException: 
                                - sigmasix=1.91.0'''
        error, pkgs = check_error(error_str=error_str, packages=packages)
        error2, pkgs2 = check_error(error_str=error_str2, packages=packages)
        error3, pkgs3 = check_error(error_str=error_str3, packages=packages)

        self.assertEqual(error, 'unresolved')
        self.assertListEqual(pkgs, ['mixpanel=1.11.0'])
        self.assertEqual(error2, 'unsatisfiable')
        self.assertListEqual(pkgs2, ['sigmasix=1.91.0'])
        self.assertIsNone(error3)
        self.assertListEqual(pkgs3, [])

    def test_extract_package_from_line(self):
        env_filename = FILES_PREFIX + 'environment.yml'
        packages = extract_env_packages(env_fname=env_filename)

        lines = [['- mixpanel=1.11.0', 'mixpanel=1.11.0'], ['sqlite<3x not compatible', 'sqlite'],
                 ['', None], ['line containing six < 3.0.x', 'six==1.11.0']]
        for line in lines:
            self.assertEqual(extract_package_from_line(line=line[0], packages=packages), line[1])


if __name__ == '__main__':
    unittest.main()



WEBAPP_URL = "https://jvn.io"
API_URL = "https://api.jovian.ai"
RC_FILENAME = ".jovianrc"
FILENAME_MSG = 'Failed to detect notebook filename. Please provide the notebook filename (including .ipynb extension) as the "nb_filename" argument to "jovian.commit".'
ISSUES_MSG = """NOTE: Jovian is current in beta, so if you face any issues, 
      please report them here: https://github.com/swiftace-ai/jovian-py/issues"""

LINUX = 'linux'
WINDOWS = 'windows'
MACOS = 'macos'

PLATFORMS = [LINUX, WINDOWS, MACOS]

API_KEY = "API_KEY"
GUEST_KEY = "GUEST_KEY"
STORAGE_ALLOWED_EXTENSIONS = tuple('ipynb yml yaml py txt csv'.split())



from requests import get, post
# from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
from os.path import basename
from time import sleep
# from tqdm import tqdm, tqdm_notebook
from jovian.utils.credentials import (read_or_request_api_key, write_api_key,
                                      CREDS, request_api_key, get_guest_key)
from jovian.utils.logger import log
from jovian.utils.jupyter import in_notebook, save_notebook, get_notebook_name
from jovian.utils.constants import API_URL, API_KEY
from jovian.utils.misc import timestamp_ms
from jovian._version import __version__


class ApiError(Exception):
    """Error class for web API related Exceptions"""
    pass


def _u(path):
    """Make a URL from the path"""
    return API_URL + path


def _msg(res):
    try:
        data = res.json()
        if 'errors' in data and len(data['errors']) > 0:
            return data['errors'][0]['message']
        if 'message' in data:
            return data['message']
        if 'msg' in data:
            return data['msg']
    except:
        if res.text:
            return res.text
        return 'Something went wrong'


def _pretty(res):
    """Make a human readable output from an HTML response"""
    return '(HTTP ' + str(res.status_code) + ') ' + _msg(res)


def validate_api_key(key):
    """Validate the API key by making a request to server"""
    res = get(_u('/user/profile'),
              headers={'Authorization': 'Bearer ' + key})
    if res.status_code == 200:
        return True
    else:
        return False
    raise ApiError(_pretty(res))


def get_api_key():
    """Retrieve and validate the API Key (from memory, config or user input)"""
    if API_KEY not in CREDS:
        key, _ = read_or_request_api_key()
        if not validate_api_key(key):
            log('The current API key is invalid or expired.', error=True)
            key, source = request_api_key(), 'request'
            if not validate_api_key(key):
                raise ApiError('The API key provided is invalid or expired.')
        write_api_key(key)
        return key
    return CREDS[API_KEY]


def _h():
    """Create authorization header with API key"""
    return {"Authorization": "Bearer " + get_api_key(),
            "x-jovian-source": "library",
            "x-jovian-library-version": __version__,
            "x-jovian-guest": get_guest_key()}


def _v(version):
    """Create version query parameter string"""
    if version is not None:
        return "?gist_version=" + str(version)
    return ""


def get_gist(slug):
    """Get the metadata for a gist"""
    res = get(url=_u('/gist/' + slug), headers=_h())
    if res.status_code == 200:
        return res.json()['data']
    raise Exception('Failed to retrieve metadata for notebook "' +
                    slug + '": ' + _pretty(res))


def get_gist_access(slug):
    """Get the access permission of a gist"""
    res = get(url=_u('/gist/' + slug + '/check-access'), headers=_h())
    if res.status_code == 200:
        return res.json()['data']
    raise Exception('Failed to retrieve access permission for notebook "' +
                    slug + '": ' + _pretty(res))


def create_gist_simple(filename=None, gist_slug=None, secret=False):
    """Upload the current notebook to create a gist"""
    auth_headers = _h()

    with open(filename, 'rb') as f:
        nb_file = (filename, f)
        log('Uploading notebook..')
        if gist_slug:
            return upload_file(gist_slug=gist_slug, file=nb_file)
        else:
            res = post(url=_u('/gist/create'),
                       data={'public': 0 if secret else 1},
                       files={'files': nb_file},
                       headers=auth_headers)
            if res.status_code == 200:
                return res.json()['data']
            raise ApiError('File upload failed: ' + _pretty(res))


def upload_file(gist_slug, file, version=None, artifact=False):
    """Upload an additional file to a gist"""
    data = {'artifact': 'true'} if artifact else {}
    res = post(url=_u('/gist/' + gist_slug + '/upload' + _v(version)),
               files={'files': file}, data=data, headers=_h())
    if res.status_code == 200:
        return res.json()['data']
    raise ApiError('File upload failed: ' + _pretty(res))


def post_blocks(blocks, version=None):
    url = _u('/data/record' + _v(version))
    res = post(url, json=blocks, headers=_h())
    if res.status_code == 200:
        return res.json()['data']
    else:
        raise ApiError('Data logging failed: ' + _pretty(res))


def post_block(data, data_type, version=None):
    """Upload metrics, hyperparameters and other information to server"""
    blocks = [{"localTimestamp": timestamp_ms(),
               "data": data,
               'recordType': data_type}]
    return post_blocks(blocks, version)


def commit_records(gist_slug, tracking_slugs, version=None):
    """Associated tracked records with a commit"""
    url = _u('/data/' + gist_slug + '/commit' + _v(version))
    res = post(url, json=tracking_slugs, headers=_h())
    if res.status_code == 200:
        return res.json()['data']
    else:
        raise ApiError('Data logging failed: ' + _pretty(res))


def post_slack_message(data, safe=False):
    """Push data to Slack, if slack is integrated with jovian account"""
    url = _u('/slack/notify')
    res = post(url, json=data, headers=_h())
    if res.status_code == 200:
        return res.json()
    elif safe:
        return {'data': {'messageSent': False}}
    else:
        raise ApiError('Slack trigger failed: ' + _pretty(res))



import platform
from jovian.utils.constants import LINUX, WINDOWS, MACOS
import time


def get_platform():
    """Identify which OS the library is running on

    Returns 'linux', 'windows' or 'macos'
    """
    system = platform.system()
    if system == 'Linux':
        return LINUX
    elif system == 'Windows':
        return WINDOWS
    elif system == 'Darwin':
        return MACOS


def timestamp_ms():
    """Return the current timestamp (in milliseconds)"""
    return int(time.time() * 1000)



import argparse
import os

from jovian.utils.clone import clone, pull
from jovian.utils.install import install, activate
from jovian.utils.slack import add_slack
from jovian._version import __version__


def exec_clone(slug, version):
    clone(slug, version)


def exec_init():
    from jovian.utils.api import get_api_key
    from jovian.utils.credentials import get_guest_key
    get_guest_key()
    get_api_key()
    print('Initialization finished')


def nb_ext(enable=True):
    if enable:
        os.system("jupyter nbextension enable jovian_nb_ext/main --sys-prefix")
    else:
        os.system("jupyter nbextension disable jovian_nb_ext/main --sys-prefix")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('command')
    parser.add_argument('gist', nargs='?')
    parser.add_argument('-n', '--name')
    parser.add_argument('-v', '--version')

    args = parser.parse_args()
    command = args.command
    if command == 'init':
        exec_init()
    elif command == 'clone':
        if not args.gist:
            print('Please provide the Gist ID to clone')
            return
        exec_clone(args.gist, args.version)
    elif command == 'pull':
        pull(args.gist, args.version)
    elif command == 'version':
        print('Jovian library version: ' + __version__)
    elif command == 'install':
        install(env_name=args.name)
    elif command == 'activate':
        activate()
    elif command == 'enable-ext':
        nb_ext()
    elif command == 'disable-ext':
        nb_ext(enable=False)
    elif command == 'add-slack':
        add_slack()


if __name__ == '__main__':
    main()
